/*=================================================================*/
/** @file D2DLL.cpp
 *  @brief Main Diablo II extra DLL handler.
 *
 *  This is the sources of the D2DLL class.
 */
/*=================================================================*/

// Some needed standard header.
#include <stdio.h>
#include <memory.h>
#include <direct.h>

// Core Class Headers
#include "Error.hpp"
#include "D2DLL.hpp"

/* Beginning of Custom Header */

#include "Items.hpp"

// Insert here all the functions header you can add to this DLL

/* End of Custom Headers */

/* Static member initialisation */
LPD2FUNCTIONTABLE D2DLL::mFunctionsList = NULL;

/*=================================================================*/
/** @brief D2DLL Wrapper constructor.
 *
 *  Nothing spicy here. You could use thios constructor to pre-calculate
 *  some constants or displaying a nice header in your DebugLog file.
 *
 */
/*=================================================================*/

D2DLL::D2DLL()
{
	char lLogFile[1024] = "";

	_getcwd( mAppPath, MAX_PATH );

	if( mAppPath[strlen(mAppPath)] != '\\')
    {
		mAppPath[strlen(mAppPath) + 1] = 0;
		mAppPath[strlen(mAppPath)] = '\\';
	}

	sprintf(lLogFile, "%sDebugLog.txt", mAppPath);

    ErrorLogger::SetLogPath( lLogFile );
    ErrorLogger::WriteInfo("Creating Diablo II DLL Wrapper ...");

    // Creating the Functions Pointers.

    mFunctionsList = new D2FUNCTIONTABLE;

    mFunctionsList->D2AddPlayerStat = (D2Common10518)0x6FDA7D70; 
    mFunctionsList->D2GetPlayerStat = (D2Common10519)0x6FDA7E40;
    mFunctionsList->D2GetRandomInt  = (D2Common10918)0x6FDA0B10;

    ErrorLogger::WriteInfo("Creating Diablo II DLL Wrapper Done.");
}

/*=================================================================*/
/** @brief D2DLL Wrapper destructor.
 *
 *  Clean up the D2DLL Wrapper and send a closing message to the debug log.
 *
 */
/*=================================================================*/

D2DLL::~D2DLL()
{
    ErrorLogger::WriteInfo("Destroying Diablo II DLL Wrapper ...");
}

/*=================================================================*/
/** @brief Module specifier.
 *
 *  Specify the HANDLE to the calling executable to be used by D2DLL.
 *
 *  @param pModule : HANDLE to the calling executable.
 *
 */
/*=================================================================*/

void D2DLL::setModule( HANDLE pModule )
{
    if( pModule != INVALID_HANDLE_VALUE )
    {
        mDLLModule = pModule;
    }
    else
    {
        ErrorLogger::ThrowError( "D2DLL::setModule()", "Invalid Module Handle used for initialization" );
    }
}

/*=================================================================*/
/** @brief D2DLL initialisation method.
 *
 *  Load up the classic D2 DLL and hook them to insert our own code.
 *
 *  @return bool : A boolean indicating if everything goes well.
 *
 */
/*=================================================================*/

bool D2DLL::init()
{
    DWORD dw = 0;

	ErrorLogger::WriteInfo( "Starting Diablo II Cabal Wars " APP_VERSION );

	if( !LoadLibrary("D2Launch.dll") )
    {
		ErrorLogger::ThrowError( "D2DLL::init()", "Failed to load library D2Launch.dll" );
	}

	if( !LoadLibrary("D2Common.dll") )
    {
		ErrorLogger::ThrowError( "D2DLL::init()", "Failed to load library D2Common.dll" );
	}

	if( !LoadLibrary("D2Game.dll") )
    {
		ErrorLogger::ThrowError( "D2DLL::init()", "Failed to load library D2Game.dll" );
	}

	if( !LoadLibrary("D2Client.dll") )
    {
		ErrorLogger::ThrowError( "D2DLL::init()", "Failed to load library D2Client.dll" );
	}

	if(!VirtualProtect((LPVOID)0x6FA11000, 0x1E000, PAGE_EXECUTE_READWRITE, &dw))
    {
		ErrorLogger::ThrowError( "D2DLL::init()", "Failed to change protection mode for D2Launch.dll" );
	}

	if(!VirtualProtect((LPVOID)0x6FD41000, 0x7C000, PAGE_EXECUTE_READWRITE, &dw))
    {
		ErrorLogger::ThrowError( "D2DLL::init()", "Failed to change protection mode for D2Common.dll" );
	}

	if(!VirtualProtect((LPVOID)0x6FC31000, 0xCF000, PAGE_EXECUTE_READWRITE, &dw))
    {
		ErrorLogger::ThrowError( "D2DLL::init()", "Failed to change protection mode for D2Game.dll" );
	}

	if(!VirtualProtect((LPVOID)0x6FAA1000, 0xCB000, PAGE_EXECUTE_READWRITE, &dw))
    {
		ErrorLogger::ThrowError( "D2DLL::init()", "Failed to change protection mode for D2Client.dll" );
	}

    __setFunctionsPointers();

    return true;
}

/*=================================================================*/
/** @brief D2DLL clean-up
 *
 *  Clean up the D2 DLL wrapper.
 *
 */
/*=================================================================*/

void D2DLL::release()
{
    ErrorLogger::WriteInfo( "Closing Diablo II DLL");
}

/*=================================================================*/
/** @brief D2DLL function intializer.
 *
 *  Write into the classic D2 DLL to specify our jump.
 *  If you ever add a new functions, think to add it here and
 *  to modify the correct D2xxx.dll location to allow it to jump correctly.
 *
 */
/*=================================================================*/

void D2DLL::__setFunctionsPointers()
{
    // Well, it looks ugly for a professional developper
    // but here we're doing something that work, not 
    // something pretty ^_^ !

    // Set up the call address for OnUseItem.
    *((int*)0x6FCFF80E) = (int)&UseItem;
}
