/*=================================================================*/
/** @file D2UnitStruct.h
 *  @brief Diablo II Unit Structures definitions.
 *
 *  (c) January 2002 - Joel Falcou for The Phrozen Keep.
 */
/*=================================================================*/

#ifndef __D2UNITSTRUCT_H__INCLUDED__
#define __D2UNITSTRUCT_H__INCLUDED__

/*=================================================================*/
/*    Unit Types enumerations.                                     */
/*=================================================================*/

enum {  UNIT_PLAYER = 0,
        UNIT_MONSTER,
	    UNIT_OBJECT,
	    UNIT_MISSILE,
	    UNIT_ITEM,
	    UNIT_ROOMTILE
     };

/*=================================================================*/
/*    Preset Unit Structures (used by DrlgRoom2).                  */
/*=================================================================*/

struct PresetUnit 
{                                   //Offset from Code
	DWORD           unitType;       //+00
	DWORD           txtFileNo;      //+04
	DWORD           unknown1[1];    //Unkonwn Data at +08
	DWORD           xPos;           //+0c
	DWORD           yPos;           //+10
	DWORD           unknown2[2];    //Unknown data at +14
	PresetUnit*     pNext;          //+1c
};


/*=================================================================*/
/*   Unit Data Structure.                                          */
/*=================================================================*/

struct UnitAny 
{                               //Offset from Code.
	DWORD   nUnitType;          //+00
    union 
    {
	    DWORD   nTxtFileNo;         
        DWORD   nPlayerClass;
    };                          //+04
	DWORD   nUnitId;            //+08
	DWORD   eMode;              //+0c
    DWORD*  unknow1;            //+10
    DWORD   nAct;               //+14
    DWORD   pActRelated;        //+18
    DWORD   unknown2[4];        //Unknown Data at +1C
    D2Seed  nSeed;              //+2C
    DWORD   startSeed;          //+34
    Path*   pPos;               //+38
	DWORD   unknown3[6];        //Unknown Data at +3C
    DWORD   animSpeed;          //+54
    DWORD   unknown4[4];        //+60
    Stats*  ptStats;            //+6C
	
    union 
    {
		PlayerData*     pPlayerData;
		MonsterData*    pMonsterData;
		ObjectData*     pObjectData;
		ItemData*       pItemData;
	};                          //+70

	DWORD   unknown5[2];        //+74
    BYTE    unknown6[2];        //+7C
    BYTE    rndDmg[2];          //+7e
    DWORD   unknown7;           //+80
	UnitInventory*  pInventory; //+84
	DWORD   unknown8[4];        //+88
    DWORD   startNull;          //+98
    DWORD   unknown9[3];        //+88
    D2Game* ptGame;             //+a4
    DWORD   unknown10[6];        //+a8
	DWORD   nOwnerType;         //+b8
	DWORD   nOwnerId;           //+bc
    DWORD   unknown11[3];       //+c0
    D2Info* ptInfo;             //+cc
    Combat* ptCombat;           //+d0
    DWORD   unknown12[5];       //+d4
    DWORD   flags;              //+e8
    DWORD   LODflag;            //+ec
	DWORD   unknown13[7];        //+f0
	UnitAny*    pNext;          //+10c
};

/*=================================================================*/
/*   Unit Specific Data Structure.                                 */
/*=================================================================*/

struct PlayerData 
{
};

struct MonsterData 
{                       // Offset from Code
};

struct ObjectData 
{                       // Offset from Code
};

struct ItemData 
{                       // Offset from Code
	DWORD nQuality;     //+00
};

/*=================================================================*/
/*   Item Location Structure.                                     */
/*=================================================================*/

struct LocInfo
{                       //Offset from Code
    DWORD noneLoc;      //+00
    DWORD headLoc;      //+04
    DWORD neckLoc;      //+08
    DWORD torsoLoc;     //+0c
    DWORD rightArmLoc;  //+10
    DWORD leftArmLoc;   //+14
    DWORD rightRingLoc; //+18
    DWORD leftRingLoc;  //+1c
    DWORD beltLoc;      //+20
    DWORD feetLoc;      //+24
    DWORD glovesLoc;    //+28
};

/*=================================================================*/
/*   Inventory Node Structure.                                     */
/*=================================================================*/

struct Node
{                   //Offset from Code.
    DWORD null;     //+00
    ItemData* pItem;    //+04
    DWORD unID;     //+08
    DWORD page;     //+0c
    Node* nextNode; //+10
};

/*=================================================================*/
/*   Inventory Structure.                                          */
/*=================================================================*/

struct UnitInventory 
{                               //Offset from Code.
    DWORD       gameRelated;    //+00
    DWORD       unit;           //+04
    DWORD       unID[2];        //+0c
    DWORD       unknown1[2];    //+14
    LocInfo*    ptLocInfo;      //+1c
    DWORD       unknown[2];     //Unknown Data at +20
    Node*       ptNode;         //+28
};

/*=================================================================*/
/*  Skill Structure.                                               */
/*=================================================================*/

struct Skills
{                           //Offset from Code.
    DWORD   skillTxt;       //+00
    Skills* pNextSkill;     //+04
    DWORD   mode;           //+08
    DWORD   unknown1[3];    //+0C
    DWORD   targetInfo;     //+18
    DWORD   targetType;     //+1c
    DWORD   targetUNID;     //+20
    DWORD   unknown2;       //+24
    DWORD   slvl;           //+28
    DWORD   slvlBonus;      //+2c
    DWORD   unknown3;       //+30
    DWORD   state;          //+34
};

/*=================================================================*/
/*  Skills Info Structure.                                         */
/*=================================================================*/

struct D2Info
{                           //Offset from Code.
    DWORD   gameRelated;    //+00
    Skills* ptSkill;        //+04
    Skills* ptLeftSkill;    //+08
    Skills* ptRightSkill;   //+0c
    Skills* ptCurrentSkill; //+10
};

struct Stats  {};
struct Combat {};
struct D2Game {};


/*=================================================================*/
/*   Unit Related Helper Functions.                                */
/*=================================================================*/

D2Game* getGamePtrFromUnit( UnitAny* pUnit );


#endif