/*=================================================================*/
/** @file Error.cpp
 *  @brief Error Logger implementation.
 *
 *  (c) January 2002 - Joel Falcou for The Phrozen Keep.
 */
/*=================================================================*/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "Common.hpp"
#include "Error.hpp"

char ErrorLogger::mErrorDesc[1024] = "";
char ErrorLogger::mErrorFile[256] = "";
char ErrorLogger::mLogFile[MAX_PATH] = "";
bool ErrorLogger::mLogInit = false;

void ErrorLogger::Exit()
{
	ErrorLogger::WriteInfo("Performing emergency shutdown under error conditions.");
	exit(0);
}

void ErrorLogger::ThrowError( const char* pLocation, const char* pDesc )
{
	strcpy( mErrorFile, pLocation);
	strcpy( mErrorDesc, pDesc);

	WriteInfo("==============================================================");
	WriteInfo("\tError trapped");
	FormatMsg("\tFile: %s", mErrorFile);
	FormatMsg("\tDescription: %s", mErrorDesc);
	WriteInfo("==============================================================");

	Exit();
}

void ErrorLogger::SetLogPath( const char* pPath )
{
	strcpy( mLogFile, pPath );

	FILE* lLog = fopen( mLogFile, "w" );

	if( lLog != NULL )
    {
	    fclose(lLog);
	    mLogInit = true;
	}
}

void ErrorLogger::WriteInfo( const char* pMessage )
{
	if( mLogInit )
    {
	    FILE* lDebug = fopen( mLogFile, "a+" );

	    if( lDebug != NULL)
        {
	        fprintf( lDebug, pMessage );
	        fprintf( lDebug, "\n" );
	        fclose( lDebug );
        }
		else
        {
            mLogInit = false;
        }
	}
    else
    {
        mLogInit = false;
    }
}

void ErrorLogger::FormatMsg( const char* pFormat, ... )
{
	if( mLogInit )
    {
	    va_list lArgs;
	    va_start( lArgs, pFormat );

	    FILE *lDebug = fopen( mLogFile, "a+" );

	    if( lDebug != NULL )
        {
	        vfprintf( lDebug, pFormat, lArgs );

	        fprintf( lDebug, "\n" );
	        fclose( lDebug );

	        va_end(lArgs);
	    }
	}
    else
    {
		mLogInit = false;
    }
}