/*=================================================================*/
/** @file Items.cpp
 *  @brief Code snippet about items.
 *
 *  (c) January 2002 - Joel Falcou for The Phrozen Keep.
 */
/*=================================================================*/

#include "Common.hpp"
#include "Error.hpp"
#include "D2DLL.hpp"
#include "Items.hpp"

/*
    This function is used to add new usable items like
    potions, elixirs, scrolls etc ...

    Additional Assembly Change in D2Game.dll v1.09b

    6FCE3073     33C0                XOR EAX,EAX
    6FCE3075     E9 90000000         JMP D2Game.6FCE310A
    6FCE307A     90                  NOP
    6FCE307B     90                  NOP
    6FCE307C     90                  NOP

    6FCE310A     > E9 F0C60100       JMP D2Game.6FCFF7FF
    6FCE310F     > 5F                POP EDI
    6FCE3110     . 5E                POP ESI
    6FCE3111     . 5D                POP EBP
    6FCE3112     . 5B                POP EBX
    6FCE3113     . 59                POP ECX
    6FCE3114     . C2 1000           RETN 10

    -----------------------------------------------

    
    6FCFF7FF     > 55                PUSH EBP
    6FCFF800     . 51                PUSH ECX
    6FCFF801     . 57                PUSH EDI
    6FCFF802     . 50                PUSH EAX
    6FCFF803     . FF15 0EF8CF6F     CALL DWORD PTR DS:[6FCFF80E]
    6FCFF809     .^E9 0139FEFF       JMP D2Game.6FCE310F
    6FCFF80E     . DEADBEEF          DD DEADBEEF

    Return 1 if you want the item to be removed, 0 if not.
*/

int STDCALL UseItem( int destroyState, UnitAny* ptItem, DWORD itemID, UnitAny* ptChar )
{
    int stat,val;
    int destroy = destroyState;

    switch( itemID )
    {
        // It would be better to use a function pointer table but ...
        // I keep it simple easy :)
        
        // Stats Elixir for STR/DEX/ENR/VIT
        case 647 :
        case 648 :
        case 649 :
        case 650 :

        stat = (itemID - 647) + STATS_STRENGTH;
        (D2DLL::getFunctions())->D2AddPlayerStat( ptChar, stat, 1 );
        destroy = 1;
        
        break;
        
        // Treasure Chest gives between 1 and 1000 gold.
        // No validation for the amount.
        case 651:
        
        val = (D2DLL::getFunctions())->D2GetRandomInt( &(ptChar->nSeed),1000 ) + 1;
        (D2DLL::getFunctions())->D2AddPlayerStat( ptChar, STATS_GOLD, val );
        destroy = 1;
        
        break;

        default: break;
    }

    return destroy;
}

